@@ -15,15 +15,15 @@
 #'     
 #' @examples
 #'     # sequence with 20 markers
-#'     mrk.seq<-make_seq_mappoly(hexafake, 1:20)
-#'     mrk.pairs<-est_pairwise_rf(input.seq = mrk.seq,
-#'                                verbose=TRUE)
+#'     mrk.seq <- make_seq_mappoly(hexafake, 1:20)
+#'     mrk.pairs <- est_pairwise_rf(input.seq = mrk.seq,
+#'                                verbose = TRUE)
 #'     ## Full recombination fraction matrix
-#'     mat<-rf_list_to_matrix(input.twopt=mrk.pairs)
+#'     mat <- rf_list_to_matrix(input.twopt = mrk.pairs)
 #'     plot(mat)
 #'     ## Matrix subset
 #'     id <- make_seq_mappoly(hexafake, 1:10)
-#'     mat.sub<-make_mat_mappoly(mat, id)
+#'     mat.sub <- make_mat_mappoly(mat, id)
 #'     plot(mat.sub)
 #'    
 #' @author Marcelo Mollinari, \email{mmollin@ncsu.edu}
@@ -37,22 +37,20 @@
 #'
 #' @export
 #' 
-make_mat_mappoly<-function(input.mat, input.seq){
+make_mat_mappoly <- function(input.mat, input.seq){
   ## checking for correct object
   input_classes1 <- c("mappoly.sequence")
   if (!inherits(input.seq, input_classes1)) {
     stop(deparse(substitute(input.seq)), " is not an object of class 'mappoly.sequence'")
   }
-  input_classes2 <-c("mappoly.rf.matrix")
+  input_classes2  <- c("mappoly.rf.matrix")
   if (!inherits(input.mat, input_classes2)) {
     stop(deparse(substitute(input.mat)), " is not an object of class 'mappoly.rf.matrix'")
   }
-  if(input.mat$cl == "poly.haplo.est.two.pts.pairwise")
-    stop("This function does not work for recombination fractions matrices originated from blocks of markers")
   input.mat$thresh.LOD.ph <- NULL
   input.mat$thresh.LOD.rf <- NULL
   input.mat$thresh.rf <- NULL
-  id <- get(input.seq$data.name, pos =1)$mrk.names[input.seq$seq.num]
+  id <- get(input.seq$data.name, pos  = 1)$mrk.names[input.seq$seq.num]
   input.mat$rec.mat <- input.mat$rec.mat[id, id]
   input.mat$lod.mat <- input.mat$lod.mat[id, id]
   input.mat
